@@ -58,8 +58,7 @@ class ArtistsView(BaseView):
         sidebar_container = Gtk.ScrolledWindow()
         sidebar_container.add(self._sidebar)
 
-        super().__init__(
-            'artists', _("Artists"), window, True, sidebar_container)
+        super().__init__('artists', _("Artists"), window, sidebar_container)
 
         self.player = player
         self._artists = {}
@@ -125,8 +124,7 @@ class ArtistsView(BaseView):
         self._view.add(new_artist_albums_widget)
 
         artist_albums = ArtistAlbumsWidget(
-            artist, albums, self.player, self._header_bar,
-            self._selection_toolbar, self._window)
+            artist, albums, self.player, self._window)
         self._artists[artist.casefold()]['widget'] = artist_albums
         new_artist_albums_widget.add(artist_albums)
         new_artist_albums_widget.show()
@@ -185,7 +183,7 @@ class ArtistsView(BaseView):
             if row.props.selected:
                 selected_artists += 1
 
-        self._update_header_from_selection(selected_artists)
+        self.props.selected_items_count = selected_artists
 
     @log
     def _on_selection_mode_changed(self, widget, data=None):
@@ -209,12 +207,10 @@ class ArtistsView(BaseView):
     @log
     def select_all(self):
         self._toggle_all_selection(True)
-        self._update_header_from_selection(len(self._sidebar))
 
     @log
     def unselect_all(self):
         self._toggle_all_selection(False)
-        self._update_header_from_selection(0)
 
     @log
     def get_selected_songs(self, callback):
