@@ -64,7 +64,7 @@ class PlaylistView(BaseView):
         sidebar_container.add(self._sidebar)
 
         super().__init__(
-            'playlists', _("Playlists"), window, True, sidebar_container)
+            'playlists', _("Playlists"), window, sidebar_container)
 
         self._window = window
         self.player = player
@@ -115,9 +115,8 @@ class PlaylistView(BaseView):
         self._sidebar.set_selection_mode(Gtk.SelectionMode.SINGLE)
         self._sidebar.connect('row-activated', self._on_playlist_activated)
 
-        self._grid.insert_column(0)
-        self._grid.child_set_property(self.stack, 'top-attach', 0)
-        self._grid.child_set_property(self.stack, 'height', 2)
+        self._grid.child_set_property(sidebar_container, 'top-attach', 0)
+        self._grid.child_set_property(sidebar_container, 'height', 2)
 
         self._iter_to_clean = None
         self._iter_to_clean_model = None
@@ -679,7 +678,7 @@ class PlaylistView(BaseView):
         if self.player.playing_playlist(
                 PlayerPlaylist.Type.PLAYLIST, playlist_id):
             self.player.stop()
-            self.set_player_visible(False)
+            self._window.set_player_visible(False)
 
         if row_next:
             self._sidebar.select_row(row_next)
