@@ -23,6 +23,8 @@
 # delete this exception statement from your version.
 
 from gettext import gettext as _
+import gi
+gi.require_version('Gd', '1.0')
 from gi.repository import Gd, Gdk, GdkPixbuf, GObject, Grl, Gtk, Pango
 
 from gnomemusic.albumartcache import Art
@@ -54,6 +56,9 @@ class SearchView(BaseView):
     def __init__(self, window, player):
         super().__init__('search', None, window)
 
+        # FIXME: Searchbar handling does not belong here.
+        self._searchbar = window._searchbar
+
         self._add_list_renderers()
         self.player = player
         self._head_iters = [None, None, None, None]
@@ -65,8 +70,7 @@ class SearchView(BaseView):
         self._albums_selected = []
         self._albums = {}
         self._albums_index = 0
-        self._album_widget = AlbumWidget(
-            player, self, self._header_bar, self._selection_toolbar)
+        self._album_widget = AlbumWidget(player, self)
         self.add(self._album_widget)
 
         self._artists_albums_selected = []
@@ -103,7 +107,7 @@ class SearchView(BaseView):
 
     @log
     def _back_button_clicked(self, widget, data=None):
-        self._header_bar.searchbar.reveal(True, False)
+        self._searchbar.reveal(True, False)
 
         if self.get_visible_child() == self._artist_albums_widget:
             self._artist_albums_widget.destroy()
@@ -113,7 +117,7 @@ class SearchView(BaseView):
                 self._window.views[View.ALBUM]._grid)
 
         self.set_visible_child(self._grid)
-        self._header_bar.props.state = HeaderBar.State.MAIN
+        self._headerbar.props.state = HeaderBar.State.MAIN
 
     @log
     def _on_item_activated(self, treeview, path, column):
@@ -136,26 +140,31 @@ class SearchView(BaseView):
             item = self.model[_iter][5]
 
             self._album_widget.update(item)
-            self._header_bar.props.state = HeaderBar.State.SEARCH
+            self._headerbar.props.state = HeaderBar.State.SEARCH
 
-            self._header_bar.props.title = title
-            self._header_bar.props.subtitle = artist
+            self._headerbar.props.title = title
+            self._headerbar.props.subtitle = artist
             self.set_visible_child(self._album_widget)
-            self._header_bar.searchbar.reveal(False)
+            self._searchbar.reveal(False)
         elif self.model[_iter][12] == 'artist':
             artist = self.model[_iter][2]
             albums = self._artists[artist.casefold()]['albums']
 
             self._artist_albums_widget = ArtistAlbumsWidget(
-                artist, albums, self.player, self._header_bar,
-                self._selection_toolbar, self._window, True)
+                artist, albums, self.player, self._window, True)
             self.add(self._artist_albums_widget)
             self._artist_albums_widget.show()
 
-            self._header_bar.props.state = HeaderBar.State.SEARCH
-            self._header_bar.props.title = artist
+            self._artist_albums_widget.bind_property(
+                'selected-items-count', self, 'selected-items-count')
+            self.bind_property(
+                'selection-mode', self._artist_albums_widget, 'selection-mode',
+                GObject.BindingFlags.BIDIRECTIONAL)
+
+            self._headerbar.props.state = HeaderBar.State.SEARCH
+            self._headerbar.props.title = artist
             self.set_visible_child(self._artist_albums_widget)
-            self._header_bar.searchbar.reveal(False)
+            self._searchbar.reveal(False)
         elif self.model[_iter][12] == 'song':
             if self.model[_iter][11] != ValidationStatus.FAILED:
                 c_iter = self._songs_model.convert_child_iter_to_iter(_iter)[1]
@@ -188,7 +197,8 @@ class SearchView(BaseView):
             iter_ = self.model.get_iter(path)
             self.model[iter_][6] = not self.model[iter_][6]
             selected_iters = self._get_selected_iters()
-            self._update_header_from_selection(len(selected_iters))
+
+            self.props.selected_items_count = len(selected_iters)
 
     @log
     def _get_selected_iters(self):
@@ -417,7 +427,7 @@ class SearchView(BaseView):
     @log
     def populate(self):
         self._init = True
-        self._header_bar.props.state = HeaderBar.State.MAIN
+        self._headerbar.props.state = HeaderBar.State.MAIN
 
     @log
     def get_selected_songs(self, callback):
