@@ -225,7 +225,8 @@ class SongsView(BaseView):
                 *event.get_coords())
             iter_ = self.model.get_iter(path)
             self.model[iter_][6] = not self.model[iter_][6]
-            self._update_header_from_selection(len(self.get_selected_songs()))
+
+            self.props.selected_items_count = len(self.get_selected_songs())
 
     @log
     def _update_model(self, player, position):
