@@ -23,10 +23,10 @@
 # delete this exception statement from your version.
 
 from gettext import ngettext
-from gi.repository import GdkPixbuf, GLib, GObject, Gtk
+from gi.repository import GdkPixbuf, GObject, Gtk
 
 from gnomemusic import log
-from gnomemusic.albumartcache import Art, ArtImage
+from gnomemusic.albumartcache import Art
 from gnomemusic.grilo import grilo
 from gnomemusic.player import PlayerPlaylist
 from gnomemusic.widgets.disclistboxwidget import DiscBox
@@ -35,7 +35,7 @@ from gnomemusic.widgets.songwidget import SongWidget
 import gnomemusic.utils as utils
 
 
-@Gtk.Template(resource_path='/org/gnome/Music/AlbumWidget.ui')
+@Gtk.Template(resource_path='/org/gnome/Music/ui/AlbumWidget.ui')
 class AlbumWidget(Gtk.EventBox):
     """Album widget.
 
@@ -48,12 +48,13 @@ class AlbumWidget(Gtk.EventBox):
     _artist_label = Gtk.Template.Child()
     _composer_label = Gtk.Template.Child()
     _composer_info_label = Gtk.Template.Child()
-    _cover = Gtk.Template.Child()
+    _cover_stack = Gtk.Template.Child()
     _disc_listbox = Gtk.Template.Child()
     _released_info_label = Gtk.Template.Child()
     _running_info_label = Gtk.Template.Child()
     _title_label = Gtk.Template.Child()
 
+    selected_items_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
 
     _duration = 0
@@ -62,37 +63,35 @@ class AlbumWidget(Gtk.EventBox):
         return '<AlbumWidget>'
 
     @log
-    def __init__(self, player, parent_view, header_bar, selection_toolbar):
+    def __init__(self, player, parent_view):
         """Initialize the AlbumWidget.
 
         :param player: The player object
         :param parent_view: The view this widget is part of
-        :param header_bar: The header bar object
-        :param selection_toolbar: The selection toolbar object
         """
         super().__init__()
 
         self._songs = []
 
+        self._cover_stack.props.size = Art.Size.LARGE
         self._parent_view = parent_view
         self._player = player
         self._iter_to_clean = None
 
         self._create_model()
-        self._album = None
-        self._header_bar = header_bar
-        self._selection_toolbar = selection_toolbar
+        self._album_name = None
 
         self.bind_property(
             'selection-mode', self._disc_listbox, 'selection-mode',
             GObject.BindingFlags.BIDIRECTIONAL)
 
         self.bind_property(
-            'selection-mode', self._header_bar, 'selection-mode',
+            'selection-mode', self._parent_view, 'selection-mode',
             GObject.BindingFlags.BIDIRECTIONAL |
             GObject.BindingFlags.SYNC_CREATE)
 
-        self.show_all()
+        self.bind_property(
+            'selected-items-count', self._parent_view, 'selected-items-count')
 
     @log
     def _create_model(self):
@@ -113,10 +112,10 @@ class AlbumWidget(Gtk.EventBox):
         )
 
     @log
-    def update(self, item):
+    def update(self, album):
         """Update the album widget.
 
-        :param item: The grilo media item
+        :param Grl.Media album: The grilo media album
         """
         # reset view
         self._songs = []
@@ -124,42 +123,33 @@ class AlbumWidget(Gtk.EventBox):
         for widget in self._disc_listbox.get_children():
             self._disc_listbox.remove(widget)
 
+        self._cover_stack.update(album)
+
         self._duration = 0
-        art = ArtImage(Art.Size.LARGE, item)
-        art.image = self._cover
 
-        self._album = utils.get_album_title(item)
-        artist = utils.get_artist_name(item)
+        self._album_name = utils.get_album_title(album)
+        artist = utils.get_artist_name(album)
 
-        self._title_label.props.label = self._album
-        self._title_label.props.tooltip_text = self._album
+        self._title_label.props.label = self._album_name
+        self._title_label.props.tooltip_text = self._album_name
 
         self._artist_label.props.label = artist
         self._artist_label.props.tooltip_text = artist
 
-        year = utils.get_media_year(item)
+        year = utils.get_media_year(album)
         if not year:
             year = '----'
         self._released_info_label.props.label = year
 
-        self._set_composer_label(item)
+        self._set_composer_label(album)
 
         self._player.connect('song-changed', self._update_model)
 
-        # If an album is playing, restore it.
-        if self._player.playing_playlist(
-                PlayerPlaylist.Type.ALBUM, self._album):
-            length = len(self._player.get_songs())
-            for i, song in enumerate(self._player.get_songs()):
-                self.add_item(None, None, song[0], length - (i + 1))
-            self.add_item(None, None, None, 0)
-            self._update_model(self._player)
-        else:
-            GLib.idle_add(grilo.populate_album_songs, item, self.add_item)
+        grilo.populate_album_songs(album, self.add_item)
 
     @log
-    def _set_composer_label(self, item):
-        composer = item.get_composer()
+    def _set_composer_label(self, album):
+        composer = album.get_composer()
         show = False
 
         if composer:
@@ -179,8 +169,7 @@ class AlbumWidget(Gtk.EventBox):
     @log
     def _on_selection_changed(self, widget):
         n_items = len(self._disc_listbox.get_selected_items())
-        self._selection_toolbar.props.items_selected = n_items
-        self._header_bar.items_selected = n_items
+        self.props.selected_items_count = n_items
 
     @log
     def _create_disc_box(self, disc_nr, disc_songs):
@@ -202,7 +191,7 @@ class AlbumWidget(Gtk.EventBox):
             return
 
         self._player.set_playlist(
-            PlayerPlaylist.Type.ALBUM, self._album, song_widget.model,
+            PlayerPlaylist.Type.ALBUM, self._album_name, song_widget.model,
             song_widget.itr)
         self._player.play()
         return True
@@ -239,7 +228,7 @@ class AlbumWidget(Gtk.EventBox):
                 self._disc_listbox.add(disc)
 
             self._set_duration_label()
-            self.show_all()
+            self._update_model(self._player)
 
     @log
     def _update_model(self, player, position=None):
@@ -248,7 +237,8 @@ class AlbumWidget(Gtk.EventBox):
         :param Player player: The main player object
         :param int position: current song position
         """
-        if not player.playing_playlist(PlayerPlaylist.Type.ALBUM, self._album):
+        if not player.playing_playlist(
+                PlayerPlaylist.Type.ALBUM, self._album_name):
             return True
 
         current_song = player.props.current_song
