@@ -34,7 +34,7 @@ from gnomemusic.widgets.songwidget import SongWidget
 logger = logging.getLogger(__name__)
 
 
-@Gtk.Template(resource_path='/org/gnome/Music/ArtistAlbumsWidget.ui')
+@Gtk.Template(resource_path='/org/gnome/Music/ui/ArtistAlbumsWidget.ui')
 class ArtistAlbumsWidget(Gtk.Box):
     """Widget containing all albums by an artist
 
@@ -47,6 +47,7 @@ class ArtistAlbumsWidget(Gtk.Box):
 
     _artist_label = Gtk.Template.Child()
 
+    selected_items_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
 
     def __repr__(self):
@@ -54,15 +55,13 @@ class ArtistAlbumsWidget(Gtk.Box):
 
     @log
     def __init__(
-            self, artist, albums, player, header_bar, selection_toolbar,
-            window, selection_mode_allowed=False):
+            self, artist, albums, player, window,
+            selection_mode_allowed=False):
         super().__init__(orientation=Gtk.Orientation.VERTICAL)
         self._player = player
         self._artist = artist
         self._window = window
         self._selection_mode_allowed = selection_mode_allowed
-        self._selection_toolbar = selection_toolbar
-        self._header_bar = header_bar
 
         self._artist_label.props.label = self._artist
 
@@ -88,11 +87,6 @@ class ArtistAlbumsWidget(Gtk.Box):
         self._songs_grid_size_group = Gtk.SizeGroup.new(
             Gtk.SizeGroupMode.HORIZONTAL)
 
-        self.bind_property(
-            'selection-mode', self._header_bar, 'selection-mode',
-            GObject.BindingFlags.BIDIRECTIONAL |
-            GObject.BindingFlags.SYNC_CREATE)
-
         self._window.notifications_popup.push_loading()
 
         self._albums_to_load = len(albums)
@@ -129,9 +123,8 @@ class ArtistAlbumsWidget(Gtk.Box):
     @log
     def _add_album(self, album):
         widget = ArtistAlbumWidget(
-            album, self._player, self._model, self._header_bar,
-            self._selection_mode_allowed, self._songs_grid_size_group,
-            self._cover_size_group)
+            album, self._player, self._model, self._selection_mode_allowed,
+            self._songs_grid_size_group, self._cover_size_group)
 
         self.bind_property(
             'selection-mode', widget, 'selection-mode',
@@ -199,8 +192,7 @@ class ArtistAlbumsWidget(Gtk.Box):
             if row[6]:
                 selected_items += 1
 
-        self._selection_toolbar.props.items_selected = selected_items
-        self._header_bar.props.items_selected = selected_items
+        self.props.selected_items_count = selected_items
 
     @log
     def select_all(self):
