@@ -160,7 +160,7 @@ class SourceManager(BaseManager):
         grilo.search_source = src
 
 
-@Gtk.Template(resource_path="/org/gnome/Music/FilterView.ui")
+@Gtk.Template(resource_path="/org/gnome/Music/ui/FilterView.ui")
 class FilterView(Gtk.TreeView):
     """TreeView for search entry items
 
@@ -254,7 +254,7 @@ class FilterView(Gtk.TreeView):
         self.props.manager.entry.emit('changed')
 
 
-@Gtk.Template(resource_path="/org/gnome/Music/DropDown.ui")
+@Gtk.Template(resource_path="/org/gnome/Music/ui/DropDown.ui")
 class DropDown(Gtk.Revealer):
     """Dropdown source/option selection widget for search
 
@@ -314,7 +314,7 @@ class DropDown(Gtk.Revealer):
         return grilo_id == "grl-tracker-source"
 
 
-@Gtk.Template(resource_path="/org/gnome/Music/Searchbar.ui")
+@Gtk.Template(resource_path="/org/gnome/Music/ui/Searchbar.ui")
 class Searchbar(Gtk.SearchBar):
     """Widget containing the search entry
     """
@@ -324,20 +324,17 @@ class Searchbar(Gtk.SearchBar):
     _search_entry = Gtk.Template.Child()
     _drop_down_button = Gtk.Template.Child()
 
+    stack = GObject.Property(type=Gtk.Stack)
+
     def __repr__(self):
         return '<Searchbar>'
 
     @log
-    def __init__(self, stack_switcher):
-        """Initialize the Searchbar
-
-        :param Gtk.Stack stack_switcher: The stack switcher containing
-          the search view.
-        """
+    def __init__(self):
+        """Initialize the Searchbar"""
         super().__init__()
 
         self._timeout = None
-        self._stack_switcher = stack_switcher
 
         self._dropdown = DropDown()
         self._dropdown.initialize_filters(self)
@@ -371,10 +368,9 @@ class Searchbar(Gtk.SearchBar):
         else:
             fields_filter = 'search_all'
 
-        stack = self._stack_switcher.props.stack
         if search_term != "":
-            stack.set_visible_child_name('search')
-            view = stack.get_visible_child()
+            self.props.stack.set_visible_child_name('search')
+            view = self.props.stack.get_visible_child()
             view.set_search_text(search_term, fields_filter)
 
         self._drop_down_button.set_active(False)
