@@ -29,7 +29,7 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-from gi.repository import Gtk, Gdk, Gio, GLib
+from gi.repository import Gtk, Gdk, Gio, GLib, GObject
 from gettext import gettext as _
 
 from gnomemusic import log
@@ -47,7 +47,9 @@ from gnomemusic.widgets.headerbar import HeaderBar
 from gnomemusic.widgets.notificationspopup import NotificationsPopup
 from gnomemusic.widgets.playertoolbar import PlayerToolbar
 from gnomemusic.widgets.playlistdialog import PlaylistDialog
+from gnomemusic.widgets.searchbar import Searchbar
 from gnomemusic.widgets.selectiontoolbar import SelectionToolbar
+from gnomemusic.windowplacement import WindowPlacement
 from gnomemusic.playlists import Playlists
 from gnomemusic.grilo import grilo
 
@@ -59,6 +61,9 @@ playlists = Playlists.get_default()
 
 class Window(Gtk.ApplicationWindow):
 
+    selected_items_count = GObject.Property(type=int, default=0, minimum=0)
+    selection_mode = GObject.Property(type=bool, default=False)
+
     def __repr__(self):
         return '<Window>'
 
@@ -66,8 +71,8 @@ class Window(Gtk.ApplicationWindow):
     def __init__(self, app):
         super().__init__(application=app, title=_("Music"))
 
-        self.settings = Gio.Settings.new('org.gnome.Music')
-        self.add_action(self.settings.create_action('repeat'))
+        self._settings = Gio.Settings.new('org.gnome.Music')
+        self.add_action(self._settings.create_action('repeat'))
         select_all = Gio.SimpleAction.new('select_all', None)
         select_all.connect('activate', self._select_all)
         self.add_action(select_all)
@@ -76,32 +81,16 @@ class Window(Gtk.ApplicationWindow):
         self.add_action(select_none)
 
         self.set_size_request(200, 100)
+        WindowPlacement(self, self._settings)
 
         self.prev_view = None
         self.curr_view = None
 
-        size_setting = self.settings.get_value('window-size')
-        if isinstance(size_setting[0], int) and isinstance(size_setting[1], int):
-            self.resize(size_setting[0], size_setting[1])
-
-        position_setting = self.settings.get_value('window-position')
-        if len(position_setting) == 2 \
-           and isinstance(position_setting[0], int) \
-           and isinstance(position_setting[1], int):
-            self.move(position_setting[0], position_setting[1])
-
-        if self.settings.get_value('window-maximized'):
-            self.maximize()
-
-        self._setup_view()
         self.notifications_popup = NotificationsPopup()
-        self._overlay.add_overlay(self.notifications_popup)
+        self._setup_view()
 
-        MediaKeys(self.player, self)
+        MediaKeys(self._player, self)
 
-        self.window_size_update_timeout = None
-        self.connect("notify::is-maximized", self._on_maximized)
-        self.connect("configure-event", self._on_configure_event)
         grilo.connect('changes-pending', self._on_changes_pending)
 
     @log
@@ -113,7 +102,7 @@ class Window(Gtk.ApplicationWindow):
                     and view_count == 1):
                 self._switch_to_player_view()
             elif (not available
-                    and not self.headerbar.props.selection_mode
+                    and not self.props.selection_mode
                     and view_count != 1):
                 self._stack.disconnect(self._on_notify_model_id)
                 self.disconnect(self._key_press_event_id)
@@ -126,33 +115,14 @@ class Window(Gtk.ApplicationWindow):
 
         grilo.songs_available(songs_available_cb)
 
-    @log
-    def _on_configure_event(self, widget, event):
-        if self.window_size_update_timeout is None:
-            self.window_size_update_timeout = GLib.timeout_add(500, self.store_window_size_and_position, widget)
-
-    @log
-    def store_window_size_and_position(self, widget):
-        size = widget.get_size()
-        self.settings.set_value('window-size', GLib.Variant('ai', [size[0], size[1]]))
-
-        position = widget.get_position()
-        self.settings.set_value('window-position', GLib.Variant('ai', [position[0], position[1]]))
-        GLib.source_remove(self.window_size_update_timeout)
-        self.window_size_update_timeout = None
-        return False
-
-    @log
-    def _on_maximized(self, klass, value, data=None):
-        self.settings.set_boolean('window-maximized', self.is_maximized())
-
     @log
     def _setup_view(self):
         self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        self.headerbar = HeaderBar()
-        self.player = Player(self)
-        self.player_toolbar = PlayerToolbar(self.player, self.headerbar)
-        self.selection_toolbar = SelectionToolbar()
+        self._headerbar = HeaderBar()
+        self._searchbar = Searchbar()
+        self._player = Player(self)
+        self._player_toolbar = PlayerToolbar(self._player, self)
+        selection_toolbar = SelectionToolbar()
         self.views = [None] * len(View)
         self._stack = Gtk.Stack(
             transition_type=Gtk.StackTransitionType.CROSSFADE,
@@ -161,6 +131,26 @@ class Window(Gtk.ApplicationWindow):
             visible=True,
             can_focus=False)
 
+        self._headerbar.bind_property(
+            "search-mode-enabled", self._searchbar, "search-mode-enabled",
+            GObject.BindingFlags.BIDIRECTIONAL |
+            GObject.BindingFlags.SYNC_CREATE)
+        self._searchbar.props.stack = self._stack
+        self._headerbar.connect(
+            'back-button-clicked', self._switch_back_from_childview)
+
+        self.connect('notify::selection-mode', self._on_selection_mode_changed)
+        self.bind_property(
+            'selected-items-count', self._headerbar, 'selected-items-count')
+        self.bind_property(
+            'selected-items-count', selection_toolbar, 'selected-items-count')
+        self.bind_property(
+            'selection-mode', self._headerbar, 'selection-mode',
+            GObject.BindingFlags.BIDIRECTIONAL |
+            GObject.BindingFlags.SYNC_CREATE)
+        self.bind_property(
+            'selection-mode', selection_toolbar, 'visible',
+            GObject.BindingFlags.SYNC_CREATE)
         # Create only the empty view at startup
         # if no music, switch to empty view and hide stack
         # if some music is available, populate stack with mainviews,
@@ -175,25 +165,24 @@ class Window(Gtk.ApplicationWindow):
         self._overlay = Gtk.Overlay()
         self._overlay.add(self._stack)
         # FIXME: Need to find a proper way to do this.
-        self._overlay.add_overlay(self.headerbar.searchbar._dropdown)
-        self.set_titlebar(self.headerbar)
-        self._box.pack_start(self.headerbar.searchbar, False, False, 0)
+        self._overlay.add_overlay(self._searchbar._dropdown)
+        self._overlay.add_overlay(self.notifications_popup)
+        self.set_titlebar(self._headerbar)
+        self._box.pack_start(self._searchbar, False, False, 0)
         self._box.pack_start(self._overlay, True, True, 0)
-        self._box.pack_start(self.player_toolbar, False, False, 0)
-        self._box.pack_start(self.selection_toolbar, False, False, 0)
+        self._box.pack_start(self._player_toolbar, False, False, 0)
+        self._box.pack_start(selection_toolbar, False, False, 0)
         self.add(self._box)
 
-        self.headerbar._search_button.connect(
+        self._headerbar._search_button.connect(
             'toggled', self._on_search_toggled)
-        self.headerbar.connect(
-            'notify::selection-mode', self._on_selection_mode_changed)
-        self.selection_toolbar.connect(
+        selection_toolbar.connect(
             'add-to-playlist', self._on_add_to_playlist)
 
-        self.headerbar.props.state = HeaderBar.State.MAIN
-        self.headerbar.show()
+        self._headerbar.props.state = HeaderBar.State.MAIN
+        self._headerbar.show()
         self._overlay.show()
-        self.player_toolbar.show_all()
+        self._player_toolbar.show_all()
         self._box.show()
         self.show()
 
@@ -203,26 +192,25 @@ class Window(Gtk.ApplicationWindow):
             else:
                 self._switch_to_empty_view()
 
-        Query()
-        if Query.music_folder:
+        if Query().music_folder:
             grilo.songs_available(songs_available_cb)
         else:
             self._switch_to_empty_view()
 
     @log
     def _switch_to_empty_view(self):
-        did_initial_state = self.settings.get_boolean('did-initial-state')
+        did_initial_state = self._settings.get_boolean('did-initial-state')
 
         if did_initial_state:
             self.views[View.EMPTY].props.state = EmptyView.State.EMPTY
         else:
             self.views[View.EMPTY].props.state = EmptyView.State.INITIAL
 
-        self.headerbar.props.state = HeaderBar.State.EMPTY
+        self._headerbar.props.state = HeaderBar.State.EMPTY
 
     @log
     def _switch_to_player_view(self):
-        self.settings.set_boolean('did-initial-state', True)
+        self._settings.set_boolean('did-initial-state', True)
         self._on_notify_model_id = self._stack.connect(
             'notify::visible-child', self._on_notify_mode)
         self.connect('destroy', self._notify_mode_disconnect)
@@ -245,15 +233,20 @@ class Window(Gtk.ApplicationWindow):
         self.views[View.SEARCH] = Gtk.Box()
 
         self.views[View.EMPTY].props.state = EmptyView.State.SEARCH
-        self.headerbar.props.state = HeaderBar.State.MAIN
-        self.headerbar.props.stack = self._stack
-        self.headerbar.searchbar.show()
+        self._headerbar.props.state = HeaderBar.State.MAIN
+        self._headerbar.props.stack = self._stack
+        self._searchbar.show()
 
-        self.views[View.ALBUM] = AlbumsView(self, self.player)
-        self.views[View.ARTIST] = ArtistsView(self, self.player)
-        self.views[View.SONG] = SongsView(self, self.player)
-        self.views[View.PLAYLIST] = PlaylistView(self, self.player)
-        self.views[View.SEARCH] = SearchView(self, self.player)
+        self.views[View.ALBUM] = AlbumsView(self, self._player)
+        self.views[View.ARTIST] = ArtistsView(self, self._player)
+        self.views[View.SONG] = SongsView(self, self._player)
+        self.views[View.PLAYLIST] = PlaylistView(self, self._player)
+        self.views[View.SEARCH] = SearchView(self, self._player)
+
+        selectable_views = [View.ALBUM, View.ARTIST, View.SONG, View.SEARCH]
+        for view in selectable_views:
+            self.views[view].bind_property(
+                'selected-items-count', self, 'selected-items-count')
 
         # empty view has already been created in self._setup_view starting at
         # View.ALBUM
@@ -269,9 +262,9 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _select_all(self, action=None, param=None):
-        if not self.headerbar.props.selection_mode:
+        if not self.props.selection_mode:
             return
-        if self.headerbar.props.state == HeaderBar.State.MAIN:
+        if self._headerbar.props.state == HeaderBar.State.MAIN:
             view = self._stack.get_visible_child()
         else:
             view = self._stack.get_visible_child().get_visible_child()
@@ -280,9 +273,9 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def _select_none(self, action=None, param=None):
-        if not self.headerbar.props.selection_mode:
+        if not self.props.selection_mode:
             return
-        if self.headerbar.props.state == HeaderBar.State.MAIN:
+        if self._headerbar.props.state == HeaderBar.State.MAIN:
             view = self._stack.get_visible_child()
             view.unselect_all()
         else:
@@ -306,33 +299,35 @@ class Window(Gtk.ApplicationWindow):
             # Open search bar on Ctrl + F
             if (keyval == Gdk.KEY_f
                     and not self.views[View.PLAYLIST].rename_active
-                    and self.headerbar.props.state != HeaderBar.State.SEARCH):
-                self.headerbar.searchbar.toggle()
+                    and self._headerbar.props.state != HeaderBar.State.SEARCH):
+                self._searchbar.toggle()
             # Play / Pause on Ctrl + SPACE
             if keyval == Gdk.KEY_space:
-                self.player.play_pause()
+                self._player.play_pause()
             # Play previous on Ctrl + B
             if keyval == Gdk.KEY_b:
-                self.player.previous()
+                self._player.previous()
             # Play next on Ctrl + N
             if keyval == Gdk.KEY_n:
-                self.player.next()
+                self._player.next()
+            if keyval == Gdk.KEY_q:
+                self.props.application.quit()
             # Toggle repeat on Ctrl + R
             if keyval == Gdk.KEY_r:
-                if self.player.props.repeat_mode == RepeatMode.SONG:
-                    self.player.props.repeat_mode = RepeatMode.NONE
+                if self._player.props.repeat_mode == RepeatMode.SONG:
+                    self._player.props.repeat_mode = RepeatMode.NONE
                     repeat_state = GLib.Variant("s", ("none"))
                 else:
-                    self.player.props.repeat_mode = RepeatMode.SONG
+                    self._player.props.repeat_mode = RepeatMode.SONG
                     repeat_state = GLib.Variant("s", ("song"))
                 self.lookup_action('repeat').change_state(repeat_state)
             # Toggle shuffle on Ctrl + S
             if keyval == Gdk.KEY_s:
-                if self.player.props.repeat_mode == RepeatMode.SHUFFLE:
-                    self.player.props.repeat_mode = RepeatMode.NONE
+                if self._player.props.repeat_mode == RepeatMode.SHUFFLE:
+                    self._player.props.repeat_mode = RepeatMode.NONE
                     repeat_state = GLib.Variant("s", ("none"))
                 else:
-                    self.player.props.repeat_mode = RepeatMode.SHUFFLE
+                    self._player.props.repeat_mode = RepeatMode.SHUFFLE
                     repeat_state = GLib.Variant("s", ("shuffle"))
                 self.lookup_action('repeat').change_state(repeat_state)
         # Ctrl+Shift+<KEY>
@@ -341,9 +336,9 @@ class Window(Gtk.ApplicationWindow):
                 self._select_none()
         # Alt+<KEY>
         elif modifiers == mod1_mask:
-            # Go back from Album view on Alt + Left
+            # Go back from child view on Alt + Left
             if keyval == Gdk.KEY_Left:
-                self.headerbar._on_back_button_clicked()
+                self._switch_back_from_childview()
             # Headerbar switching
             if keyval in [Gdk.KEY_1, Gdk.KEY_KP_1]:
                 self._toggle_view(View.ALBUM)
@@ -357,16 +352,16 @@ class Window(Gtk.ApplicationWindow):
         else:
             if (keyval == Gdk.KEY_AudioPlay
                     or keyval == Gdk.KEY_AudioPause):
-                self.player.play_pause()
+                self._player.play_pause()
 
             if keyval == Gdk.KEY_AudioStop:
-                self.player.stop()
+                self._player.stop()
 
             if keyval == Gdk.KEY_AudioPrev:
-                self.player.previous()
+                self._player.previous()
 
             if keyval == Gdk.KEY_AudioNext:
-                self.player.next()
+                self._player.next()
 
             child = self._stack.get_visible_child()
             if (keyval == Gdk.KEY_Delete
@@ -374,21 +369,21 @@ class Window(Gtk.ApplicationWindow):
                 self.views[View.PLAYLIST].remove_playlist()
             # Close selection mode or search bar after Esc is pressed
             if keyval == Gdk.KEY_Escape:
-                if self.headerbar.props.selection_mode:
-                    self.headerbar.props.selection_mode = False
+                if self.props.selection_mode:
+                    self.props.selection_mode = False
                 else:
-                    self.headerbar.searchbar.reveal(False)
+                    self._searchbar.reveal(False)
 
         # Open the search bar when typing printable chars.
         key_unic = Gdk.keyval_to_unicode(keyval)
-        if ((not self.headerbar.searchbar.get_search_mode()
+        if ((not self._searchbar.get_search_mode()
                 and not keyval == Gdk.KEY_space)
                 and GLib.unichar_isprint(chr(key_unic))
                 and (modifiers == shift_mask
                      or modifiers == 0)
                 and not self.views[View.PLAYLIST].rename_active
-                and self.headerbar.props.state != HeaderBar.State.SEARCH):
-            self.headerbar.searchbar.reveal(True)
+                and self._headerbar.props.state != HeaderBar.State.SEARCH):
+            self._searchbar.reveal(True)
 
     @log
     def do_button_release_event(self, event):
@@ -399,11 +394,11 @@ class Window(Gtk.ApplicationWindow):
         __, code = event.get_button()
         # Mouse button 8 is the navigation button
         if code == 8:
-            self.headerbar._on_back_button_clicked()
+            self._switch_back_from_childview()
 
     @log
     def _notify_mode_disconnect(self, data=None):
-        self.player.stop()
+        self._player.stop()
         self.notifications_popup.terminate_pending()
         self._stack.disconnect(self._on_notify_model_id)
 
@@ -418,14 +413,9 @@ class Window(Gtk.ApplicationWindow):
                     or self.prev_view == self.views[View.EMPTY])):
             self.curr_view.set_visible_child(self.curr_view._grid)
 
-        # Slide out sidebar on switching to Artists or Playlists view
-        if self.curr_view == self.views[View.ARTIST] or \
-           self.curr_view == self.views[View.PLAYLIST]:
-            self.curr_view.stack.set_visible_child_name('dummy')
-            self.curr_view.stack.set_visible_child_name('sidebar')
         if (self.curr_view != self.views[View.SEARCH]
                 and self.curr_view != self.views[View.EMPTY]):
-            self.headerbar.searchbar.reveal(False)
+            self._searchbar.reveal(False)
 
         # Disable the selection button for the EmptySearch and Playlist
         # view
@@ -434,7 +424,7 @@ class Window(Gtk.ApplicationWindow):
             self.views[View.PLAYLIST]
         ]
         allowed = self.curr_view not in no_selection_mode
-        self.headerbar.props.selection_mode_allowed = allowed
+        self._headerbar.props.selection_mode_allowed = allowed
 
         # Disable renaming playlist if it was active when leaving
         # Playlist view
@@ -448,20 +438,20 @@ class Window(Gtk.ApplicationWindow):
         # the search mode. This fixes the behaviour as needed, but is
         # incorrect: searchview currently does not switch states
         # correctly.
-        if (not self.headerbar.props.selection_mode
-                and not self.headerbar.props.state == HeaderBar.State.CHILD
-                and not self.headerbar.props.state == HeaderBar.State.SEARCH):
+        if (not self.props.selection_mode
+                and not self._headerbar.props.state == HeaderBar.State.CHILD
+                and not self._headerbar.props.state == HeaderBar.State.SEARCH):
             self._stack.set_visible_child(self.views[view_enum])
 
     @log
     def _on_search_toggled(self, button, data=None):
-        self.headerbar.searchbar.reveal(
+        self._searchbar.reveal(
             button.get_active(), self.curr_view != self.views[View.SEARCH])
         if (not button.get_active()
                 and (self.curr_view == self.views[View.SEARCH]
                     or self.curr_view == self.views[View.EMPTY])):
             child = self.curr_view.get_visible_child()
-            if self.headerbar.props.state == HeaderBar.State.MAIN:
+            if self._headerbar.props.state == HeaderBar.State.MAIN:
                 # We should get back to the view before the search
                 self._stack.set_visible_child(
                     self.views[View.SEARCH].previous_view)
@@ -470,12 +460,31 @@ class Window(Gtk.ApplicationWindow):
                     and child != self.curr_view._artist_albums_widget):
                 self._stack.set_visible_child(self.views[View.ALBUM])
 
-            if self.headerbar.props.selection_mode:
-                self.headerbar.props.selection_mode = False
+            if self.props.selection_mode:
+                self.props.selection_mode = False
+
+    @log
+    def _switch_back_from_childview(self, klass=None):
+        if self.props.selection_mode:
+            return
+
+        views_with_child = [
+            self.views[View.ALBUM],
+            self.views[View.SEARCH]
+        ]
+        if self.curr_view in views_with_child:
+            self.curr_view._back_button_clicked(self.curr_view)
+
+        if self.curr_view != self.views[View.SEARCH]:
+            self._searchbar.reveal(False)
 
     @log
     def _on_selection_mode_changed(self, widget, data=None):
-        if self.headerbar.props.selection_mode == False:
+        if self.props.selection_mode:
+            self._player_toolbar.hide()
+        elif self._player.props.playing:
+            self._player_toolbar.show()
+        if not self.props.selection_mode:
             self._on_changes_pending()
 
     @log
@@ -492,7 +501,7 @@ class Window(Gtk.ApplicationWindow):
             if playlist_dialog.run() == Gtk.ResponseType.ACCEPT:
                 playlists.add_to_playlist(
                     playlist_dialog.get_selected(), selected_songs)
-            self.headerbar.props.selection_mode = False
+            self.props.selection_mode = False
             playlist_dialog.destroy()
 
         self._stack.get_visible_child().get_selected_songs(callback)
@@ -503,4 +512,4 @@ class Window(Gtk.ApplicationWindow):
 
         :param bool visible: actionbar visibility
         """
-        self.player_toolbar.set_visible(visible)
+        self._player_toolbar.set_visible(visible)
